;Mini-Shell Enhanced by Chicane

;version 2.0 Lite (scrolling version)

#include "asm86.h"

#include "ti86asm.inc"

#include "ti86ops.inc"



_load_ram_ahl	equ		$462F	;Converts ahl to ASIC, loads RAM page.

_homeup			equ		$4A95	;Resets _curRow to 0,0.

_exec_basic		equ		$4C57	;Execute TI-BASIC program in OP1.

_exec_asm		equ		$5730	;Execute z80 assembly program in OP1.

_runoff			equ		$4AB1	;Disable run indicator.

_runon			equ		$4AAD	;Enable run indicator.

_checkbasic		equ		$46AB	;Check program in OP1 for TI-BASIC.

_checkasm		equ		$46AF	;Check program in OP1 for z80 assembly.

_getcode		equ		$4068	;Get scancode, but 1000's of clocks faster than GET_KEY.



.org _asm_exec_ram
ASMstart:
	call	_runoff			;Turn off the busy indicator.

	call	_clrScrn		;Clear the screen and _textshadow.
	ld		hl,0
	ld		(NumProgs),hl
SetupGUI:
	ld		hl,$0001		;First row, second column.

	ld		(_penCol), hl

	ld		hl,PrgmName		;Load the text offset into hl.

	call	_vputs			;Output in vfont.

	ld		a,$6C

	ld		(_penCol),a

	call	_vputs
	ld		hl,VIDEO_MEM	

	call	Invert


FindProgs:
	call	$47F3
	ld		hl,AVAT
	ld		(Pointer2AVAT),hl
	ld		hl,$BFFF
SearchLoop:
	ld 		de,(VAT_END)
	inc 	de
	or 		a
	sbc 	hl,de
	jp 		c,FoundAll
	add 	hl,de
	ld 		a,(hl)
	cp		$12
	jr		nz,NotAProg
	push	hl
	call	ProgCheck
	pop		hl
	jr		nz,ProgFound
NotAProg:
 	dec 	hl
	dec 	hl
	dec 	hl
	dec 	hl
	dec 	hl
 	ld 		b,(hl)
 	inc 	b
SkipName:
	dec 	hl
	djnz 	SkipName
	jr 		SearchLoop

ProgCheck:
	ld		de,-6
	add		hl,de
	ld		a,(hl)
	cp		$10		;sqrt program
	ret		z
	cp		'#'		;a system var marked as a progra,
	ret		z
	cp		'!'		;also a system var
	ret		z
	cp		'm'		;if the first letter isnt m return
	ret		nz
	dec		hl
	ld		a,(hl)
	cp		's'		;if the second letter isnt s return
	ret		nz
	dec		hl
	ld		a,(hl)
	cp		'e'		;if the third letter isnt e, return
	ret		nz
	ret

ProgFound:
	dec 	hl				
	dec 	hl
	dec 	hl
	dec 	hl
	dec 	hl
	ld 		b,(hl)
	ld 		ix,(Pointer2AVAT)
	ld 		a,$12
	ld 		(ix),a
	inc		ix
	ld		a,b
	ld		(ix),a
	inc		ix
SaveName:
	dec 	hl
	ld 		a,(hl)
	ld 		(ix),a
	inc 	ix
	djnz 	SaveName
	ld 		(ix),$0			;End of inverted Name
	dec 	hl
	ld 		(Pointer2VAT),hl
	ld		hl,(Pointer2AVAT)
	ld		de,12
	add		hl,de
	ld		(Pointer2AVAT),hl
	ld		hl,(Pointer2VAT)
	ld		a,(NumProgs)
	inc		a
	ld		(NumProgs),a
	jp		SearchLoop

FoundAll:
	ld		hl,(Pointer2AVAT)
	ld		de,-12
	add		hl,de
	ld		(Pointer2End),hl
	ld		hl,AVAT
	ld		(Pointer2AVAT),hl
	call	Move12b
	ld		a,(NumProgs)
	cp		0
	jp		z,NoProgs

Display8Progs:
	ld		a,1
	ld		(Pos),a
	ld		hl,(Pointer2AVAT)
TheMoveLoop:
	call	Move12b
	ld		(TempPointer),hl
	call	DisplayHandler
	ld		hl,(TempPointer)
	ld		a,(Pos)
	inc		a
	ld		(Pos),a
	cp		9
	jr		z,DisplayDone
	ld		de,(Pointer2End)
	sbc		hl,de
	jr		nc,DisplayDone
	ld		hl,(TempPointer)
	jr		TheMoveLoop	
DisplayDone:
	ld		hl,(Pointer2AVAT)
	call	Move12b
	ld		a,1

	ld		(Pos),a

	call	SetBlack
	call	DisplayHandler
	call	SetWhite


GetKey1:
	call	_getcode

	cp		K_EXIT

	jr		z,Exit

	cp		K_UP

	jr		z,ProgUp

	cp		K_DOWN

	jp		z,ProgDown

	cp		K_SECOND

	jr		z,ExecPrgm

	cp		K_PLUS

	jr		z,ContrastUp

	cp		K_MINUS

	jr		z,ContrastDn

	jr		GetKey1



ExecPrgm:
	rst 	10h

	ld		a,b

	ex		de,hl

	call	_load_ram_ahl

	inc		hl

	inc		hl

	xor		a

	cp		(hl)

	jr		z,Chk_Hex

	inc		hl

	ld		a,(hl)

	cp		$27					;tokenized hex

	jr		z,ExecAsm

	cp		$29					;protected basic

	jr		z,ExecBasic

	cp		$28					;assembly

	jr		z,ExecAsm



Chk_Hex:						;If the first byte is $00 and the 5th is 'P', then it is a non-tokenized Hex program

	inc		hl

	inc		hl

	inc		hl

	inc		hl

	ld		a,'P'				;now the 5th byte

	cp		(hl)

	jr		z,ExecAsm			;if it isnt an asmprgm prog, then its basic



ExecBasic:
	call	_runon

	call	_exec_basic

	jp		ASMstart



ExecAsm:
	call	_exec_asm

	jp		ASMstart



Exit:
	call	_clrScrn

	ret

NoProgs
	call	_clrScrn			;clear screen

	call	_homeup				;reset _curRow etc

	ret


ContrastDn:
	ld		a,-1

	jr		ContrastMain

ContrastUp:
	ld		a,1

ContrastMain:
	ld		hl,CONTRAST

	add		a,(hl)

	and		$1F

	ld		(hl),a

	out		(2),a

	jr		GetKey1



;***********************Move Up Routines*************************

ProgUp:
	ld		hl,(Pointer2AVAT)
	ld		de,AVAT
	sbc		hl,de
	jp		z,GetKey1
	call	DisplayHandler

	ld		hl,(Pointer2AVAT)
	ld		de,-12
	add		hl,de
	ld		(Pointer2AVAT),hl
	call	Move12b	


MoveUp:
	ld      hl,Pos

	ld      a,$01

	cp      (hl)

	call    z,NoMoveUp

	dec     (hl)

	call	SetBlack
	call	DisplayHandler

	call	SetWhite
	jp		GetKey1



NoMoveUp:
	push	hl

	ld		hl,$FF84

	ld		de,$FFF4

	ld		c,48

ScrollScreenDn:
	ld		b,5

LooperDn:
	ld		a,(hl)

	ld		(de),a

	dec		hl

	dec		de

	djnz 	LooperDn

	push	de

	ld		de,-11

	add		hl,de

	pop		de

	push	hl

	ld		hl,-11

	add		hl,de

	ex		de,hl

	pop		hl

	dec		c

	ld		a,c

	or		a

	jr		nz,ScrollScreenDn

	pop		hl

	inc		(hl)

	ret


;**************************Move Down Routines**************************

ProgDown:
	ld		hl,(Pointer2AVAT)
	ld		de,(Pointer2End)
	sbc		hl,de
	jp		z,GetKey1
	call	DisplayHandler
	ld		hl,(Pointer2AVAT)
	ld		de,12
	add		hl,de
	ld		(Pointer2AVAT),hl
	ld		de,_OP1
	call	Move12b


MoveDown:
	ld      hl,Pos

	ld      a,8

	cp      (hl)

	ld		a,$01

	call    z,NoMoveDown

	inc     (hl) 

	call	SetBlack
	call	DisplayHandler

	call	SetWhite
	jp		GetKey1



NoMoveDown:
	push	hl

	ld		hl,$FCF0

	ld		de,$FC80

	ld		c,48

ScrollScreenUp:
	ld		b,5

LooperUp:
	ld		a,(hl)

	ld		(de),a

	inc		hl

	inc		de

	djnz 	LooperUp

	push	de

	ld		de,11

	add		hl,de

	pop		de

	push	hl

	ld		hl,11

	add		hl,de

	ex		de,hl

	pop		hl

	dec		c

	ld		a,c

	or		a

	jr		nz,ScrollScreenUp

	pop		hl

	dec		(hl)

	ret


DisplayHandler:
	ld		a,(Pos)

	ld		h,a

	ld		l,$07

	call	MUL_HL

	ld		de,$01

	add		hl,de

	ld		(_penRow),hl

	call	ClearLine

	ld		a,$0

	ld		(_penCol),a

	ld		hl,Space_Str
	call	_vputs
	ld		a,(_OP1+1)
	ld		b,a
	ld		hl,_OP1+2
	call	_vputsn
	ld		hl,Space_Str
	call	_vputs
	ret


SetBlack:
	set		3,(iy+5)

	set		1,(iy+5)

	ret

SetWhite:
	res		3,(iy+5)

	ret


ClearLine:
	ld      a,(Pos)				;load the position 1-7

	ld      h,a					;into h

	ld      l,$70				;

	call    MUL_HL				;

	ld      de,$FC10			;add offset

	add     hl,de			

	ld		c,$06

ClearLines:
	push	hl

	call	Clearer	

	pop		hl

	ld		de,$10

	add		hl,de

	dec		c

	ld		a,c

	or		a

	jr		nz,ClearLines

	ret
Clearer:
	ld		b,$05

ClearLoop:
	ld		(hl),$00

	inc		hl

	djnz 	ClearLoop

	ret

Move12b:
	ld		bc,12
	ld		de,_OP1
	ldir
	ret


Invert:
	ld      b,$70

InvertLoop:
	ld      a,(hl)

	cpl
	ld      (hl),a

	inc     hl

	djnz	InvertLoop

	ret


;********************Strings used in program*********************

Space_Str:
	.db "   ",0


PrgmName:
	.db "MSE Lite",0



Version:
	.db "v2.0",0



;--------------Vars used in program------>

Pos:
	.db	0

NumProgs:
	.dw 0000

TempPointer:
	.dw 0000

Pointer2End:
	.dw 0000

Pointer2VAT:
	.dw 0000

Pointer2AVAT:
	.dw 0000
AVAT:
.end
